@@ -18,10 +18,16 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
+/**********************************************************************
+ * Functions:
+ **********************************************************************/
+
+error_reporting(0) ; 
 require_once ("functions.php"); 
 loadConfig () ; 
 session_start () ; 
 
+
 require('/usr/share/php/smarty/libs/Smarty.class.php') ; 
 
 loadLocale () ;
@@ -31,6 +37,78 @@ $smarty = new Smarty();
 $smarty->template_dir = $smarty_templ ; 
 $smarty->compile_dir = $smarty_compile ; 
 
+// user defined error handling function
+function userErrorHandler($errno, $errmsg, $filename, $linenum, $vars) 
+{
+    global $ldap, $error_log_file, $error_mail_to, $error_show, $ldap_err, $ldap_msg ; 
+
+    // timestamp for the error entry
+    $dt = date("Y-m-d H:i:s (T)");
+
+    // define an assoc array of error string
+    // in reality the only entries we should
+    // consider are E_WARNING, E_NOTICE, E_USER_ERROR,
+    // E_USER_WARNING and E_USER_NOTICE
+    $errortype = array (
+                E_ERROR              => 'Error',
+                E_WARNING            => 'Warning',
+                E_PARSE              => 'Parsing Error',
+                E_NOTICE             => 'Notice',
+                E_CORE_ERROR         => 'Core Error',
+                E_CORE_WARNING       => 'Core Warning',
+                E_COMPILE_ERROR      => 'Compile Error',
+                E_COMPILE_WARNING    => 'Compile Warning',
+                E_USER_ERROR         => 'User Error',
+                E_USER_WARNING       => 'User Warning',
+                E_USER_NOTICE        => 'User Notice',
+                E_STRICT             => 'Runtime Notice',
+                E_RECOVERABLE_ERRROR => 'Catchable Fatal Error'
+                );
+    // set of errors for which a var trace will be saved
+    $user_errors = array(E_USER_ERROR, E_USER_WARNING, E_USER_NOTICE);
+    
+     
+    $err = "<errorentry>\n";
+    $err .= "\t<datetime>" . $dt . "</datetime>\n";
+    $err .= "\t<errornum>" . $errno . "</errornum>\n";
+    $err .= "\t<errortype>" . $errortype[$errno] . "</errortype>\n";
+    $err .= "\t<errormsg>" . $errmsg . "</errormsg>\n";
+    $err .= "\t<scriptname>" . $filename . "</scriptname>\n";
+    $err .= "\t<scriptlinenum>" . $linenum . "</scriptlinenum>\n";
+
+    if (in_array($errno, $user_errors)) {
+        $err .= "\t<vartrace>" . wddx_serialize_value($vars, "Variables") . "</vartrace>\n";
+    }
+
+    $err .= "</errorentry>\n\n";
+    if (strpos($errmsg, "ldap_") === 0) {
+        $ldap_err = ldap_errno ($ldap) ; 
+        switch ($ldap_err) {
+            case 50:
+                $ldap_msg = "I'm sorry, but you dont have the propper access to do this" ; 
+                break ;
+            default:
+                echo "<pre>\n" ; 
+		echo "ldap_err: " . $ldap_err . "\n" ;  
+		echo "err_msg: " . ldap_err2str ($ldap_err) . "\n" ; 
+                echo "</pre>\n" ; 
+		break ; 
+        }
+    }
+
+    if ($error_show) 
+	echo "<pre>\n" . $err . "</pre>\n" ; 
+
+    // save to the error log, and e-mail me if there is a critical user error
+    if (!empty($error_log_file))
+	error_log($err, 3, "/tmp/lwat.err");
+    if ($errno == E_USER_ERROR && !empty($error_mail_to)) {
+        mail($error_mail_to, "Critical User Error", $err);
+    }
+}
+
+$old_error_handler = set_error_handler("userErrorHandler");
+
 function autoMount ($viewbase) {
     global $smarty, $ldap, $automountbase ; 
 
@@ -120,7 +198,7 @@ function automountdetail ($dn) {
     $smarty->display ('admin_automount_edit.tpl') ; 
 }
 
-function showHost ($ldap, $authenticated, $cn) {
+function showHost ($ldap, $cn) {
     global $smarty, $hostbase, $base ; 
     $filter="cn=" . $cn ;
     $want = array ("cn", "associatedDomain", "ipHostNumber", "macAddress", "sambaAcctFlags") ;
@@ -144,40 +222,34 @@ function showHost ($ldap, $authenticated, $cn) {
 	$smarty->assign ('sambaHost', $sambaHost) ;
 	$smarty->assign ('memberOf', $memberOf) ; 
 	$smarty->assign ('notMemberOf', $notMemberOf) ; 
-	showHostGroups ($ldap, $authenticated, $cn) ; 
+	showHostGroups ($ldap, $cn) ; 
 	$smarty->display('admin_host_show.tpl') ;
     } else {
         printf (_("No matching objects") ."<BR>") ;
     }
 }
 
-function showHostGroups ($ldap, $authenticated, $cn) {
+function showHostGroups ($ldap, $cn) {
     global $smarty, $netgroupbase ;
 
     $filter="(&(objectClass=nisNetGroup)(!(memberNisNetGroup=*)))" ; 
-	$want = array ("cn", "nisNetGroupTriple") ;
+    $want = array ("cn", "nisNetGroupTriple") ;
     $searchbase=$netgroupbase ; 
-        $result = ldap_search($ldap, $searchbase, $filter, $want);
-        if ($result) {
-	    $entries = ldap_get_entries ($ldap, $result);
-	    for ($i = 0 ; $i < $entries["count"] ; $i++) {
-	    if (@in_array ('(' . $cn . ',-,-)', $entries[$i]["nisnetgrouptriple"]))
-	           $memberOf[] = htmlspecialchars($entries[$i]['cn'][0]) ; 
-	       else
-	           $notMemberOf[] = htmlspecialchars($entries[$i]['cn'][0]) ; 
-            }
-        } 
-    if ($authenticated) {
-        $groupTarget='\'sub\'' ;
-        $delAction ='\'hostDelGroup&cn=' . $cn ; 
-        $delAction .= '&group=\' + member.value' ;
-        $addAction ='\'hostAddGroup&cn=' . $cn ; 
-        $addAction .= '&group=\' + nonmember.value';
-    } else {
-        $groupTarget = '\'logout\'' ; 
-        $delAction = '\'loginForm\'' ; 
-        $addAction = '\'loginForm\'' ; 
-    }
+    $result = ldap_search($ldap, $searchbase, $filter, $want);
+    if ($result) {
+	$entries = ldap_get_entries ($ldap, $result);
+	for ($i = 0 ; $i < $entries["count"] ; $i++) {
+	if (@in_array ('(' . $cn . ',-,-)', $entries[$i]["nisnetgrouptriple"]))
+	       $memberOf[] = htmlspecialchars($entries[$i]['cn'][0]) ; 
+	   else
+	       $notMemberOf[] = htmlspecialchars($entries[$i]['cn'][0]) ; 
+	}
+    } 
+    $groupTarget='\'sub\'' ;
+    $delAction ='\'hostDelGroup&cn=' . $cn ; 
+    $delAction .= '&group=\' + member.value' ;
+    $addAction ='\'hostAddGroup&cn=' . $cn ; 
+    $addAction .= '&group=\' + nonmember.value';
     $smarty->assign('memberOf', $memberOf) ; 
     $smarty->assign('notMemberOf', $notMemberOf) ; 
     $smarty->assign('groupTarget', $groupTarget) ; 
@@ -227,7 +299,7 @@ function hostAddGroup ($ldap, $cn, $group) {
 	    $change[] = htmlspecialchars($entries[0]["nisnetgrouptriple"][$i]) ;
 	}
 	$change[] = "(" . $cn . ",-,-)" ; 
-	ldap_modify($ldap, $groupdn, array ("nisNetGroupTriple" => $change)) ;
+	@ldap_modify($ldap, $groupdn, array ("nisNetGroupTriple" => $change)) ;
     }
 }
 
@@ -245,7 +317,7 @@ function hostDelGroup ($ldap, $cn, $group) {
 	if ($entries[0]["nisnetgrouptriple"][$i] <>  '(' . $cn . ',-,-)')
 	    $change[] = htmlspecialchars($entries[0]["nisnetgrouptriple"][$i]) ;
     }
-    ldap_modify($ldap, $groupdn, array ("nisNetGroupTriple" => $change)) ;
+    @ldap_modify($ldap, $groupdn, array ("nisNetGroupTriple" => $change)) ;
 }
 
 function changeGroup ($ldap, $gid, $cn, $description, $auth) {
@@ -261,7 +333,7 @@ function changeGroup ($ldap, $gid, $cn, $description, $auth) {
     if ($description <> $entries[0]["description"][0]) {
 	$change["description"][] = $description ; 
     }
-    if (count ($change)) ldap_modify($ldap, $groupdn, $change) ;
+    if (count ($change)) @ldap_modify($ldap, $groupdn, $change) ;
     $change = array (); 
     if (!$useLisGroup) {
 	$filter="(&(member=*)(cn=" . $cn . "))";
@@ -283,7 +355,7 @@ function changeGroup ($ldap, $gid, $cn, $description, $auth) {
 		    }
 		}
                 if  (count($change["member"]) > 0)
-                    ldap_add ($ldap, "cn=" . $cn . "," . $authbase, $change) ;
+                    @ldap_add ($ldap, "cn=" . $cn . "," . $authbase, $change) ;
             }
         } else {
             if ($count > 0) {
@@ -344,7 +416,7 @@ function changeHost ($ldap, $cn, $domain, $ipaddress, $macaddress, $sambaHost =
 	    if (!empty ($change["objectClass"]))
 		for ($i = 0 ; $i < $entries[0]["objectclass"][count]; $i++)
 		    $change["objectClass"][] = $entries[0]["objectclass"][$i] ; 
-	    if (!ldap_modify($ldap, "cn=" . $cn . "," . $hostbase, $change))
+	    if (!@ldap_modify($ldap, "cn=" . $cn . "," . $hostbase, $change))
 	        debug (array('entries' => $entries, 'change' => $change)) ;
 	}
     }
@@ -375,7 +447,7 @@ function addHost ($ldap, $cn, $domain, $ipaddress, $macaddress) {
 	    $add["objectClass"][] = "ieee802Device" ; 
 	    $add["macAddress"] = $macaddress ;
 	}
-	ldap_add($ldap, "cn=" . $cn . "," . $hostbase, $add) ;
+	@ldap_add($ldap, "cn=" . $cn . "," . $hostbase, $add) ;
     }
     showHost ($ldap, true, $cn);
 }
@@ -401,22 +473,23 @@ function addGroup ($ldap, $cn, $description) {
 	$add["cn"] = $cn ; 
 	$add["gidNumber"] = $gidNumber ;
 	$add["description"] = $description ; 
-	ldap_add($ldap, "cn=" . $cn . "," . $groupbase, $add) ;
+	@ldap_add($ldap, "cn=" . $cn . "," . $groupbase, $add) ;
     }
     showGroup ($ldap, $gidNumber);
 }
 
 function addUser ($ldap, $cn, $template, $username) {
     $userpw = pwgen () ;
-    ldapAddUser ($ldap, $cn, $template, $username, $userpw, $groups) ; 
+    if (ldapAddUser ($ldap, $cn, $template, $username, $userpw, $groups)) {
+        printf(_("Added user: %s") . '<BR>', $cn) ;
+	printf(_("username: ") . 
+	       '<A HREF="javascript:;" 
+		onClick="Action(\'showUser&text=%s\', \'main\')">%s</A><BR>', 
+		$username, $username) ; 
+	printf (_("password: %s"), $userpw);
+        printf ("<HR>\n") ; 
+    }
     userAddForm (true) ;
-    printf ("<HR>\n") ; 
-    printf(_("Added user: %s") . '<BR>', $cn) ;
-    printf(_("username: ") . 
-           '<A HREF="javascript:;" 
-            onClick="Action(\'showUser&text=%s\', \'main\')">%s</A><BR>', 
-            $username, $username) ; 
-    printf (_("password: %s"), $userpw);
 }
 
 function changeUser ($ldap, $uid, $cn) {
@@ -427,7 +500,7 @@ function changeUser ($ldap, $uid, $cn) {
     $entries = ldap_get_entries ($ldap, $result);
     $change = array (); 
     $change["cn"][] = $cn ; 
-    ldap_modify($ldap, $entries[0]["dn"], $change) ; 
+    @ldap_modify($ldap, $entries[0]["dn"], $change) ; 
     showUser ($ldap, true, $uid); 
 }
 
@@ -475,7 +548,7 @@ function showGroup ($ldap, $gid) {
 
 }
 
-function showUser ($ldap, $authenticated, $uid) {
+function showUser ($ldap, $uid) {
     global $smarty, $base, $groupbase ; 
     $filter="(&(objectClass=posixAccount)(uid=" . $uid . "))";
     $want = array ("uidNumber", "uid", "cn", "gidNumber") ;
@@ -497,15 +570,15 @@ function showUser ($ldap, $authenticated, $uid) {
 	$smarty->assign('uidNumber', $uidNumber) ;  
 	$smarty->assign('cn', $cn) ;  
 	$smarty->assign('userGroup', $userGroup) ;  
-	showUserGroups($ldap, $authenticated, $uid) ; 
+	showUserGroups($ldap, $uid) ; 
 	$smarty->display ('admin_user_show.tpl') ;  
     } else {
         printf (_("No matching objects") ."<BR>") ;
     }
 }
 
-function showUserGroups ($ldap, $authenticated ,$uid) {
-    global $smarty, $useLisGroup, $groupbase ;
+function showUserGroups ($ldap, $uid) {
+    global $smarty, $useLisGroup, $groupbase, $base ;
 
     if ($useLisGroup) 
 	$filter="(&(objectClass=posixGroup)(!(groupType=private))(memberUid=" . $uid . "))";
@@ -528,20 +601,21 @@ function showUserGroups ($ldap, $authenticated ,$uid) {
     $result = ldap_search($ldap, $searchbase, $filter, $want);
     $entries = ldap_get_entries ($ldap, $result);
     for ($i=0 ; $i < $entries["count"] ; $i++) {
+        if (!$useLisGroup && empty($entries[$i]['memberUid'])) {
+            $filter = "(&(objectClass=posixAccount)(gidNumber=" . 
+                        $entries[$i]["gidnumber"][0] . "))" ; 
+            $result = ldap_search ($ldap, $base, $filter, array ('dn')) ; 
+            if (ldap_count_entries ($ldap, $result) > 0) 
+                continue ; 
+        }
         $nonMemberValues[] = htmlspecialchars($entries[$i]["gidnumber"][0]) ;
         $nonMemberDisplays[] = htmlspecialchars($entries[$i]["cn"][0]) ;
     }
-    if ($authenticated) {
-        $groupTarget='\'sub\'' ;
-        $delAction ='\'userDelGroup&uid=' . $uid ; 
-        $delAction .= '&gid=\' + member.value' ;
-        $addAction ='\'userAddGroup&uid=' . $uid ; 
-        $addAction .= '&gid=\' + nonmember.value';
-    } else {
-        $groupTarget = '\'logout\'' ; 
-        $delAction = '\'loginForm\'' ; 
-        $addAction = '\'loginForm\'' ; 
-    }
+    $groupTarget='\'sub\'' ;
+    $delAction ='\'userDelGroup&uid=' . $uid ; 
+    $delAction .= '&gid=\' + member.value' ;
+    $addAction ='\'userAddGroup&uid=' . $uid ; 
+    $addAction .= '&gid=\' + nonmember.value';
     $smarty->assign('memberValues', $memberValues) ; 
     $smarty->assign('memberDisplays', $memberDisplays) ; 
     $smarty->assign('nonMemberValues', $nonMemberValues) ; 
@@ -556,6 +630,9 @@ function showResult ($ldap, $type, $text, $searchDisabled) {
     global $smarty, $useLisGroup, $base, $groupbase, $hostbase ; 
 
     if ($text == "*") $text="" ; 
+    $replace = array ("(", ")", "\\", "\0") ; 
+    $with = array ("0x28", "0x29", "0x5c", "0x00") ; 
+    $text = str_replace ($replace, $with, $text) ; 
     switch ($type) {
         case "User":
             if ($searchDisabled == 'true')
@@ -574,7 +651,7 @@ function showResult ($ldap, $type, $text, $searchDisabled) {
                 $filter="(&(objectClass=posixGroup)(!(groupType=private))(cn=" . $text . "*))" ;
             else 
                 $filter="(&(objectClass=posixGroup)(cn=" . $text . "*))" ;
-	    $want = array ("gidNumber", "cn") ;
+	    $want = array ("gidNumber", "cn", "memberUid") ;
 	    $searchbase=$groupbase ; 
 	    $action="showGroup" ; 
 	    $show="gidnumber" ; 
@@ -595,6 +672,18 @@ function showResult ($ldap, $type, $text, $searchDisabled) {
     if ($count > 50)
         printf (_("Too many entries, please narrow your search") . "<BR>\n") ; 
     elseif ($count) {
+        for ($i=0 ; $i <= $count && $type == "Group" ; $i++) {
+            if (empty($entries[$i]['memberuid'])) {
+                $filter="(&(objectClass=posixAccount)(gidNumber=" .
+                          $entries[$i]['gidnumber'][0] . "))" ; 
+		$result = ldap_search ($ldap, $base, $filter, array ('dn')) ; 
+		if (ldap_count_entries ($ldap, $result) > 0) {
+		    array_splice($entries, $i + 1, 1) ; 
+		    $i-- ; 
+		    $count-- ; 
+		}
+	    }
+        }
 	for ($i=0 ; $i < $count ; $i++) {
 	    $value[]=htmlspecialchars($entries[$i][$show][0]) ; 
 	    if ($type == "User") 
@@ -616,97 +705,51 @@ function showResult ($ldap, $type, $text, $searchDisabled) {
     }
 }
 
-function userAddForm ($authenticated) {
+function userAddForm () {
     global $smarty ; 
     
     $templates = parse_ini_file ("/etc/lwat/admin.ini", true); 
     foreach ($templates as $name => $fields) 
         $templname[] = $name ; 
     $smarty->assign ('templates',$templname) ;
-    if ($authenticated) {
-        $target = '\'main\'' ; 
-	$action = '\'addUser&cn=\' + cn.value '; 
-	$action .= ' + \'&username=\' + username.value' ;
-	$action .= ' + \'&template=\' + template.value' ;
-    } else {
-	$action = '\'loginForm\'' ; 
-        $target = '\'logout\'' ; 
-    }
+    $target = '\'main\'' ; 
+    $action = '\'addUser&cn=\' + cn.value '; 
+    $action .= ' + \'&username=\' + username.value' ;
+    $action .= ' + \'&template=\' + template.value' ;
     $smarty->assign ('target', $target) ;  
     $smarty->assign ('action', $action) ;  
     $smarty->display ('admin_user_add.tpl') ; 
 }
 
+$key = lwat_key () ; 
 
-$ldap=ldap_connect ($ldaphost); 
-$authenticated = ($_COOKIE['Authenticated'] == "True") ;
-$key = $_SESSION['key'] ;
-if ($key == "") {
-    if ($authenticated) {
-        setcookie ("Authenticated", "False", time () + 1800) ;
-    }
-    $authenticated = false ; 
-} 
-ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
-@ldap_start_tls($ldap) ; 
-if ($authenticated) {
-    $admindn = $_COOKIE['admindn'] ; 
-    $xorstring = $_COOKIE['xorstring'] ; 
-    if ($xorstring <> "") {
-	$adminpw = xorstring ($key, $xorstring) ; 
-	$authenticated = ($adminpw <> "") ;
-    } else 
-        $authenticated = false ; 
-}
-if ($authenticated) {
-    $bind = @ldap_bind ($ldap, $admindn, $adminpw); 
-    if ($bind) {
-	setcookie ("Authenticated", "True", time () + 1800) ;
-	setcookie ("admindn", $admindn, time () + 1800) ;
-	setcookie ("xorstring", $xorstring, time () + 1800) ;
-    } else {
-	unset ($_SESSION['key']) ; 
-	$bind = @ldap_bind ($ldap) ;
-	$authenticated = false ; 
-	setcookie ('Authenticated', 'false', time () + 1800) ;
-	setcookie ('admindn', 'dummy', time() + 1800) ;
-	setcookie ('xorstring', 'dummy', time() + 1800) ;
-	unset ($_SESSION['key']) ; 
-	$smarty->assign ('authenticated', 'false') ;
-    }
-} else {
-    unset ($_SESSION['key']) ; 
-    $bind = @ldap_bind ($ldap) ;
-}
+$ldap=lwat_ldap (); 
+if (lwat_login ($ldap, '', '', $key)) 
+    $action = readkey ('action') ; 
+else 
+    $action = 'logout';
 
-switch (readKey('action')) {
+switch ($action) {
+    case "logout": 
+	$smarty->display ('admin_relogin.tpl') ; 
+        break ;
     case "userAddForm":
-        userAddForm ($authenticated) ;
+        userAddForm () ;
         break ;
     case "groupAddForm":
-	if ($authenticated) {
-	    $action ='\'addGroup&cn=\' + cn.value '; 
-	    $action .= ' + \'&description=\' + description.value' ;
-	    $target = '\'main\'' ; 
-	} else {
-	    $action = '\'loginForm\'' ; 
-	    $target = '\'logout\'' ; 
-	}
+	$action ='\'addGroup&cn=\' + cn.value '; 
+	$action .= ' + \'&description=\' + description.value' ;
+	$target = '\'main\'' ; 
 	$smarty->assign ('action', $action) ;  
 	$smarty->assign ('target', $target) ;  
 	$smarty->display ('admin_group_add.tpl') ; 
         break ;
     case "machineAddForm":
-	if ($authenticated) {
-	    $action ='\'addHost&cn=\' + cn.value '; 
-	    $action .= ' + \'&domain=\' + domain.value' ;
-	    $action .= ' + \'&ipaddress=\' + ipaddress.value' ;
-	    $action .= ' + \'&macaddress=\' + macaddress.value' ;
-	    $target = '\'main\'' ; 
-	} else {
-	    $action = '\'loginForm\'' ; 
-	    $target = '\'logout\'' ; 
-	}
+	$action ='\'addHost&cn=\' + cn.value '; 
+	$action .= ' + \'&domain=\' + domain.value' ;
+	$action .= ' + \'&ipaddress=\' + ipaddress.value' ;
+	$action .= ' + \'&macaddress=\' + macaddress.value' ;
+	$target = '\'main\'' ; 
 	$smarty->assign ('action', $action) ;  
 	$smarty->assign ('target', $target) ;  
 	$smarty->display ('admin_host_add.tpl') ; 
@@ -730,30 +773,30 @@ switch (readKey('action')) {
         $smarty->display('admin_user_search.tpl') ;
         break ;
     case "showUser":
-        showUser ($ldap, $authenticated, readKey('text')); 
+        showUser ($ldap, readKey('text')); 
         break ;
     case "userDelGroup":
         $uid = readKey('uid') ;
         userDelGroup ($ldap, $uid, readKey('gid')); 
-        showUserGroups ($ldap, $authenticated, $uid);
+        showUserGroups ($ldap, $uid);
 	$smarty->display('admin_user_show_group.tpl') ; 
         break ;
     case "hostDelGroup":
         $cn = readKey('cn') ;
         hostDelGroup ($ldap, $cn, readKey('group')); 
-        showHostGroups ($ldap, $authenticated, $cn);
+        showHostGroups ($ldap, $cn);
 	$smarty->display('admin_host_show_group.tpl') ; 
         break ;
     case "hostAddGroup":
         $cn = readKey('cn') ;
         hostAddGroup ($ldap, $cn, readKey('group')); 
-        showHostGroups ($ldap, $authenticated, $cn);
+        showHostGroups ($ldap, $cn);
 	$smarty->display('admin_host_show_group.tpl') ; 
         break ;
     case "userAddGroup":
         $uid = readKey('uid') ;
         userAddGroup ($ldap, $uid, readKey('gid')); 
-        showUserGroups ($ldap, $authenticated, $uid);
+        showUserGroups ($ldap, $uid);
 	$smarty->display('admin_user_show_group.tpl') ; 
         break ;
     case "changeUser":
@@ -855,7 +898,7 @@ switch (readKey('action')) {
             $enable['shadowFlag'] = '0' ; 
             $enable['loginShell'] = '/bin/bash' ; 
             $enable['sambaAcctFlags'] = '[U          ]' ; 
-	    ldap_modify ($ldap, $value, $enable);
+	    @ldap_modify ($ldap, $value, $enable);
         }
         break ; 
     case "userDisable":
@@ -908,7 +951,7 @@ switch (readKey('action')) {
             $disable['shadowFlag'] = '1' ; 
             $disable['loginShell'] = 'DISABLED!/bin/bash' ; 
             $disable['sambaAcctFlags'] = '[DU         ]' ; 
-	    ldap_modify ($ldap, $value, $disable);
+	    @ldap_modify ($ldap, $value, $disable);
         }
         break ; 
     case "userDelete":
@@ -987,11 +1030,9 @@ switch (readKey('action')) {
 	               $entries[0]['gidnumber'][0] . "))" ; 
 	    $result = ldap_search ($ldap, $base, $filter, $want);
 	    $entries = ldap_get_entries ($ldap, $result);
-	    if ($entries[count]) {
-		if ($entries[0]["memberuid"][count] == 0)
-		    ldap_delete ($ldap, $entries[0]["dn"]) ; 
-		ldap_delete ($ldap, $value) ; 
-	    }
+	    if ($entries[0]["memberuid"][count] === 0)
+		ldap_delete ($ldap, $entries[0]["dn"]) ; 
+	    ldap_delete ($ldap, $value) ; 
         }
         break ; 
     case "addGroup":
@@ -1039,13 +1080,15 @@ switch (readKey('action')) {
 		$hostIP = gethostbyname ($cn . "." . $domain . ".") ;
         }
         if ($domain == "") {
-	    $fullname = strtolower(gethostbyaddr ($hostIP)) ;
+	    $fullname = @strtolower(gethostbyaddr ($hostIP)) ;
 	    $pos = strpos ($fullname, $cn) ; 
 	    if ($pos === false)
 	        $domain = "" ;
 	    else 
 	        $domain = substr ($fullname, strlen($cn) + 1) ;
 	}
+	if ($hostIP == $cn)
+	    $hostIP = "" ; 
 	printf ('<table><tr><td>'); 
 	printf ('<input type=text name="domain" value="%s"></td>', $domain) ; 
 	printf ('<td><input type=text name="ipaddress" value="%s">', $hostIP) ; 
@@ -1084,67 +1127,37 @@ switch (readKey('action')) {
 		$info["sambaLMPassword"][] = $sambahash->lmhash($newpw) ; 
 		$info["sambaNTPassword"][] = $sambahash->nthash($newpw);
 	    }
-	    $result = ldap_modify ($ldap,$entries[0]["dn"],$info) ;
-	    printf(_("Changed password for %s ( %s ) to %s ."),$entries[0]['uid'][0],$entries[0]['cn'][0], $newpw );
+	    $result = @ldap_modify ($ldap,$entries[0]["dn"],$info) ;
+	    if ($result) 
+		printf(_("Changed password for %s ( %s ) to %s ."),
+		          $entries[0]['uid'][0],$entries[0]['cn'][0], $newpw );
+	    else {
+		 $ldaperr = ldap_errno ($ldap) ; 
+		 switch ($ldaperr) {
+		     case 50:
+		         printf (_("Sorry, you are not allowed to change password for %s (%s)"), 
+		                 $entries[0]['uid'][0], $entries[0]['cn'][0]) ; 
+		         break ;
+		     default:
+		          reportLdapError ($ldaperr, "Changing password") ;
+		  }
+	    }
 	}
-        showUser ($ldap, $authenticated, $entries[0]['uid'][0]); 
-	break ;
-    case "logout":
-	setcookie ('Authenticated', 'false', time () + 1800) ;
-	setcookie ('admindn', 'dummy', time() + 1800) ;
-	setcookie ('xorstring', 'dummy', time() + 1800) ;
-	unset ($_SESSION['key']) ; 
-	$smarty->assign ('authenticated', 'false') ;
-	$smarty-> display ('admin_login.tpl') ; 
+        showUser ($ldap, $entries[0]['uid'][0]); 
 	break ;
     case "newPassAsk":
         $uid=readKey('uid') ; 
-        if ($authenticated) {
-            printf ('<form method=post name=newpw>') ;
-	    printf (_("Are you sure you want to set a new password?") . '<BR>') ;
-	    printf (_("Suggested password: ")) ; 
-	    printf ('<INPUT type=text name="newpw" value="%s" %s><BR>', 
-	             pwgen(), ($allowPwSet ? "" : "readonly") ) ; 
-	    printf ('<input type=button name=yes value="' . _("Yes") . 
-	            '" onClick="Action(\'newPassSet&uid=%s&newpw=\' + newpw.value, \'main\');">', $uid) ;
-	    printf ('<input type=button name=no value="' . _("No") . 
-	            '" onClick="Action(\'showUser&text=%s\', \'main\');">', $uid) ;
-	    printf ('</form>') ; 
-        } else {
-	    printf (_("You must log in before setting a new password") . '<BR>') ;
-	    showUserGroups($ldap, $authenticated, $uid) ; 
-	    $smarty->display('admin_user_show_group.tpl') ; 
-	}
+	printf ('<form method=post name=newpw>') ;
+	printf (_("Are you sure you want to set a new password?") . '<BR>') ;
+	printf (_("Suggested password: ")) ; 
+	printf ('<INPUT type=text name="newpw" value="%s" %s><BR>', 
+		 pwgen(), ($allowPwSet ? "" : "readonly") ) ; 
+	printf ('<input type=button name=yes value="' . _("Yes") . 
+		'" onClick="Action(\'newPassSet&uid=%s&newpw=\' + newpw.value, \'main\');">', $uid) ;
+	printf ('<input type=button name=no value="' . _("No") . 
+		'" onClick="Action(\'showUser&text=%s\', \'main\');">', $uid) ;
+	printf ('</form>') ; 
 	break ; 
-    case "login":
-        $loginbase = $base ; 
-        $adminUser=readKey('admindn') ;
-        $filter = '(|(&(objectClass=posixAccount)(uid=' . $adminUser . '))(&(objectClass=simpleSecurityObject)(cn=' . $adminUser . ')))' ; 
-        $want = array ("dn") ; 
-        while (!empty ($loginbase) && !$authenticated) {
-	    $result = @ldap_search($ldap, $loginbase, $filter, $want);
-	    $entries = @ldap_get_entries ($ldap, $result); 
-	    $admindn = htmlspecialchars($entries[0]['dn']) ; 
-	    $adminpw=readKey('adminpw') ;
-	    $bind = @ldap_bind ($ldap, $admindn, $adminpw); 
-	    if ($bind) {
-		$key = crypt(sprintf (gettimeofday (true))) ;
-		$_SESSION['key'] = $key ; 
-		$xorstring= xorstring ($key, $adminpw); 
-		setcookie ('Authenticated', 'True', time () + 1800) ;
-		setcookie ('admindn', $admindn, time () + 1800) ;
-		setcookie ('xorstring', $xorstring, time () + 1800) ;
-		$smarty->assign('authenticated', 'true') ; 
-		$authenticated = true ; 
-	    } else {
-		$loginbase = ldap_explode_dn ($loginbase,0) ;
-		array_splice ($loginbase, 0,2) ; 
-		$loginbase = implode (",", $loginbase) ; 
-            }
-        }
-    case "loginForm":
-        $smarty->display('admin_login.tpl') ; 
-        break ;
     case "machineSearchForm":
         $smarty->display('admin_machine_search.tpl') ;
         break ;
@@ -1153,7 +1166,7 @@ switch (readKey('action')) {
         if ($searchFor <> "") showResult ($ldap, 'Hosts', $searchFor, ''); 
         break ;
     case "showHost":
-        showHost ($ldap, $authenticated, readKey('text')); 
+        showHost ($ldap, readKey('text')); 
         break ;
     case "hostsDelete":
         $hostToDelete = array () ; 
@@ -1259,12 +1272,12 @@ switch (readKey('action')) {
 	            }
                 $info["automountInformation"] = sprintf ("ldap:%s", $newbase) ; 
                 $dn = sprintf("cn=%s,ou=auto.master,%s", $add, $automountbase) ;
-                ldap_add ($ldap, $dn, $info) ;
+                @ldap_add ($ldap, $dn, $info) ; 
                 unset ($info) ; 
                 $info["objectClass"] = array ("top", "organizationalUnit") ; 
                 $info["ou"] = $lastou ; 
                 $info["description"] = sprintf ("holder for %s mount point", $add) ; 
-                ldap_add ($ldap, $newbase, $info) ; 
+                @ldap_add ($ldap, $newbase, $info) ; 
                 $dn = sprintf ("ou=%s,%s", substr($add,1), $automountbase) ; 
             } elseif (empty($export) && empty ($options)) {
                 $info["objectClass"] = Array("top","Automount"); 
@@ -1276,13 +1289,13 @@ switch (readKey('action')) {
                 for ($i = $display[count] ; $i-- > 0 ; )
                     $displaystr = sprintf ("%s/%s", $displaystr, $display[$i]) ;
                 $info["Description"] = sprintf ("%s submount point", $displaystr) ; 
-		ldap_add ($ldap, $dn, $info) ; 
+		@ldap_add ($ldap, $dn, $info) ; 
 		unset ($info) ;
                 $info["objectClass"] = Array("top","organizationalUnit"); 
                 $info["ou"] = $add ; 
                 $info["Description"] = sprintf ("holder for %s submount point", $displaystr) ; 
                 $dn = sprintf("ou=%s,%s", $add, $root) ;
-		ldap_add ($ldap, $dn, $info) ; 
+		@ldap_add ($ldap, $dn, $info) ; 
             } elseif (empty($export) || empty($options))
                 printf (_("Sorry, you need to enter both an exportpath and options to define a automount share")) ; 
             else {
@@ -1294,7 +1307,7 @@ switch (readKey('action')) {
                 for ($i = $display[count] ; $i-- > 0 ; )
                     $displaystr = sprintf ("%s/%s", $displaystr, $display[$i]) ;
                 $info["Description"] = sprintf ("%s mount point", $displaystr) ; 
-		ldap_add ($ldap, $dn, $info) ; 
+		@ldap_add ($ldap, $dn, $info) ; 
             } 
 	    autoMount ($dn) ; 
         } else {
