@@ -21,16 +21,29 @@
     loadConfig () ; 
     require('/usr/share/php/smarty/libs/Smarty.class.php') ; 
     session_start() ;
-    $authenticated = ($_COOKIE['Authenticated'] == "True") ;
     loadLocale () ; 
     ob_start ();
-
     $smarty = new Smarty();
+
     $smarty->template_dir = $smarty_templ ; 
     $smarty->compile_dir = $smarty_compile ; 
 
-    if ($authenticated)
-        $smarty->assign ('authenticated', 'true') ; 
-    $smarty->display('index.tpl');
+    $authenticated = false ; 
+    $key = lwat_key () ; 
+    $ldap = lwat_ldap () ; 
 
+    if ($_GET['action'] == 'logout') {
+	setcookie ('admindn', '', time () + 1800) ;
+	setcookie ('xorstring', '', time () + 1800) ;
+    } else 
+        $authenticated = lwat_login ($ldap, 
+                                     readKey('admindn'), 
+                                     readKey('adminpw'),
+                                     $key) ; 
+
+    if ($authenticated) 
+	$smarty->display('menu.tpl');
+    else
+	$smarty->display('login.tpl');
+    ldap_close ($ldap) ; 
 ?>
